% The following will produce graphs of the relative error as a function of
%    acceleration, a histogram of the relative error, and of relative
%    populations as a function of acceleration for each pair of range and
%    number of samples.

% The variables and code use unitless variables which are detailed in
%    Appendix A.
v = 2;           % Velocities of split lattice
T = 320;         % Total cycle time
alpha = 10/32;   % Lattice strength
nm = 7;          % Number of harmonics to use (goes from -nm to nm)

samples = [10,50]; % Number of samples generated for neural network
g_ranges = [[1 5]; % Ranges of acceleration (as multiples of g)
            [2 10]];
n=1;
for hng = 1:length(g_ranges)
    % Cycles through one range at a time.
    g_range = g_ranges(hng,:); % Range of acc to use for training
    
    disp(['>Range is from ',num2str(g_range(1)),'g to ',num2str(g_range(2)),'g'])
    
    for i = 1:length(samples)
        dg = (g_range(2)-g_range(1))/samples(i); 
        % If the start of the range is 0, then it shifts it over.
        % If your range starts at a negative values and then includes zero,
        %   further cases need to be aded, but this was unneccesary for our
        %   work.
        if g_range(1) == 0
            g_range(1) = dg;
            samples(i) = samples(i)-1;
        end
        g_range2 = [g_range(1)+dg/2 g_range(2)-dg/2]; % Range of acc for testing
        [Acc,Pops] = CounterProp(samples(i),v,T,g_range,alpha,nm);
        net = fitnet(32,'trainbr');
        net.divideParam.trainRatio = 1;
        net.divideParam.valRatio = 0;
        net.divideParam.testRatio = 0;
        net.trainParam.showWindow = false;
        net = train(net,Pops,Acc); 
        [Acc,Pops] = CounterProp(samples(i)-1,v,T,g_range2,alpha,nm);
        
        z = net(Pops);
        figure(n);
        errors = (abs(Acc-z)./Acc);
        histogram(errors,'Normalization','probability');
        ylabel('Relative bin count');
        xlabel('Relative error');
        title(['Histogram of relative error for range from ',num2str(g_range(1)),'$g$ to ',num2str(g_range(2)),'$g$ for ',num2str(samples(i)),' points'],'Interpreter','LaTeX');
        
        figure(n+1);
        plot(Acc,errors);
        disp(['For ',num2str(samples(i)),' samples, the mean is ',num2str(mean(errors)),' and the median is ',num2str(median(errors)),'.'])
        xlabel('Acceleration (m/s)');
        ylabel('Relative Error');
        title(['Relative Error for range from ',num2str(g_range(1)),'$g$ to ',num2str(g_range(2)),'$g$ using ',num2str(samples(i)),' samples'],'Interpreter','LaTeX');
        
        figure(n+2);
        imagesc(Acc,-5:5,Pops);
        colorbar;
        xlabel('Acceleration (m/s)');
        ylabel('Plane wave harmonic');
        title('Plane wave harmonic populations vs Acceleration');
        
        n=n+3;
    end
end